Imports System.Runtime.InteropServices
Imports System.DirectoryServices
Imports System.Collections.Specialized

Module GetNetListModule
    'NET_API_STATUS NetServerEnum(
    '  LPCWSTR servername,
    '  DWORD level,
    '  LPBYTE* bufptr,
    '  DWORD prefmaxlen,
    '  LPDWORD entriesread,
    '  LPDWORD totalentries,
    '  DWORD servertype,
    '  LPCWSTR domain,
    '  LPDWORD resume_handle
    ');
    Declare Unicode Function NetServerEnum Lib "Netapi32.dll" ( _
        ByVal servername As Integer, _
        ByVal level As Integer, _
        ByRef bufptr As IntPtr, _
        ByVal prefmaxlen As Integer, _
        ByRef entriesread As Integer, _
        ByRef totalentries As Integer, _
        ByVal servertype As Integer, _
        ByVal domain As String, _
        ByRef resume_handle As Integer) As Integer

    'NET_API_STATUS NetApiBufferFree(
    '  LPVOID Buffer
    ');
    Declare Function NetApiBufferFree Lib "Netapi32.dll" ( _
        ByVal Buffer As IntPtr) As Integer

    Const SV_TYPE_DOMAIN_CTRL As Integer = &H8
    Const SV_TYPE_DOMAIN_BAKCTRL As Integer = &H10
    Const SV_TYPE_SERVER_NT As Integer = &H8000
    Const SV_TYPE_ALL As Integer = &HFFFFFFFF

    'typedef struct _SERVER_INFO_101 {
    '  DWORD sv101_platform_id;
    '  LPWSTR sv101_name;
    '  DWORD sv101_version_major;
    '  DWORD sv101_version_minor;
    '  DWORD sv101_type;
    '  LPWSTR sv101_comment;
    '} SERVER_INFO_101, *PSERVER_INFO_101, *LPSERVER_INFO_101;
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Unicode, Pack:=4)> _
    Structure SERVER_INFO_101
        Dim sv101_platform_id As Integer
        Dim sv101_name As String
        Dim sv101_version_major As Integer
        Dim sv101_version_minor As Integer
        Dim sv101_type As Integer
        Dim sv101_comment As String
    End Structure

    Function GetNetList(ByVal DomainName As String, ByVal Filter As String, Optional ByVal Svr_Only As Boolean = False) As String()
        Dim ServerInfo As New SERVER_INFO_101
        Dim i, j, MaxLenPref, level, ret, EntriesRead, TotalEntries, ResumeHandle As Integer
        Dim iPtr, BufPtr As IntPtr
        Dim ans(-1) As String
        Dim mask As Integer

        ' let's do it!
        MaxLenPref = -1
        level = 101

        If Svr_Only Then
            mask = SV_TYPE_SERVER_NT Or SV_TYPE_DOMAIN_CTRL Or SV_TYPE_DOMAIN_BAKCTRL
        Else
            mask = SV_TYPE_ALL
        End If
        ret = NetServerEnum(0, level, BufPtr, MaxLenPref, EntriesRead, TotalEntries, mask, DomainName, ResumeHandle)
        If ret <> 0 Then
            MsgBox("Hmmmm... something went wrong" & vbCrLf & "Can't get list of PCs from " & DomainName, vbCritical, "Error!")
            Return ans
        End If

        ReDim ans(EntriesRead - 1)

        ' loop thru the entries
        j = 0
        For i = 0 To EntriesRead - 1
            ' copy the stuff into our structure
            iPtr = New IntPtr(BufPtr.ToInt32 + (Len(ServerInfo) * i))
            ServerInfo = CType(Marshal.PtrToStructure(iPtr, GetType(SERVER_INFO_101)), SERVER_INFO_101)

            If Filter = "" Or ServerInfo.sv101_name = Filter.ToUpper Or ServerInfo.sv101_name Like Filter.ToUpper Then
                ' add it to our string array
                ans(j) = ServerInfo.sv101_name
                j += 1
            End If
        Next
        NetApiBufferFree(BufPtr)

        ReDim Preserve ans(j - 1)
        Return ans
    End Function

    Public Function GetPCOUList(ByVal DomainName As String, ByVal OU_Name As String, ByVal Filter As String, ByVal Svr_Only As Boolean) As String()
        Dim entry, de As DirectoryEntry
        Dim srch As DirectorySearcher
        Dim result As SearchResult
        Dim ldap, buf, os, ans(-1) As String
        Dim temp As New StringCollection

        Try
            ldap = "LDAP://" & DomainName & "/" & OU_Name

            ' set our starting place to search
            entry = New DirectoryEntry(ldap)
            srch = New DirectoryServices.DirectorySearcher(entry)
            srch.Filter = "(ObjectCategory=computer)"

            ' OK, let's do it
            For Each result In srch.FindAll()
                de = result.GetDirectoryEntry

                buf = de.Properties("CN").Value
                If Filter = "" Or Filter.ToUpper = buf.ToUpper Or buf.ToUpper Like Filter.ToUpper Then
                    os = de.Properties("operatingSystem").Value
                    If Not IsNothing(os) Then
                        If Svr_Only = False Or (Svr_Only = True And os Like "*Server*") Then
                            temp.Add(buf)
                        End If
                    End If
                End If
            Next
        Catch
        End Try

        If temp.Count > 0 Then
            ReDim ans(temp.Count - 1)
            temp.CopyTo(ans, 0)
            [Array].Sort(ans)
        End If
        Return ans
    End Function
End Module
